<?php
/**
 * DouPHP
 * --------------------------------------------------------------------------------------------------
 * 版权所有 2013-2024 漳州豆壳网络科技有限公司，并保留所有权利。
 * 网站地址: http://www.douphp.com
 * --------------------------------------------------------------------------------------------------
 * 这不是一个自由软件！您只能在遵守授权协议前提下对程序代码进行修改和使用；不允许对程序代码以任何形式任何目的的再发布。
 * 授权协议: http://www.douphp.com/license.html
 * --------------------------------------------------------------------------------------------------
 * Author: DouCo Co.,Ltd.
 * Release Date: 2024-09-30
 */
define('IN_DOUCO', true);

$sub = 'checkout|cashier|offlinepay|success';
$subbox = array(
        "module" => 'order',
        "sub" => $sub
);

$_CUR = 'order';
require (dirname(__FILE__) . '/include/init.php');

// 权限验证
$callback = $dou_user->check_login_state(); // 登录验证
if ($callback['msg'] == 'ok') {
    $user_id = $callback['user_id'];
} else {
    $data['wrong'] = $_LANG['login_timeout'];
    die(json_encode($data));
}

// 引入和实例化订单功能
include_once (ROOT_PATH . 'include/order.class.php');
$dou_order = new Order();

// rec操作项的初始化
$rec = $check->is_rec($_REQUEST['rec']) ? $_REQUEST['rec'] : 'default';

/**
 * +----------------------------------------------------------
 * 购物车项目列表
 * +----------------------------------------------------------
 */
if ($rec == 'default') {
    $cart = $dou_order->get_cart($user_id);
 
    // 数据封装
    $data['title'] = $_LANG['order_cart'];
    $data['cart'] = $cart;
}

/**
 * +----------------------------------------------------------
 * 项目加入购物车
 * +----------------------------------------------------------
 */
elseif ($rec == 'insert') {
    // 初始化
    $data['wrong'] = '';
 
    $POST = get_object_vars(json_decode(str_replace("\\", "", $_REQUEST['post'])));
 
    $module = $check->is_extend_id($POST['module']) ? $POST['module'] : 'product';
    $item_id = $check->is_number($POST['item_id']) ? $POST['item_id'] : '';
    $item_number = $check->is_number($POST['item_number']) ? $POST['item_number'] : 1;
    $mode = $check->is_letter($_POST['mode']) ? $_POST['mode'] : 'money';
 
    // 判断项目是否存在
    if (!$dou->value_exist($module, 'id', $item_id)) {
        // 错误提示
        $data['wrong'] = $_LANG['order_item_wrong'];
        die(json_encode($data));
    }
 
    // 如果购物车中存在不同模块商品或为积分兑换，则先清空购物车
    if ($dou->value_exist('order_cart', 'user_id', $user_id, "module != '$module'") || $mode == 'point')
        $dou_order->clear_cart($user_id);
 
    // 项目信息
    $item = $dou->get_row($module, '*', "id = '$item_id'");
    $name = $item['name'] ? $item['name'] : $item['title'];
    $price = $item['price'];
 
    // 款式属性
    if ($_OPEN['attribute']) {
        include_once (ROOT_PATH . 'include/attribute.class.php');
        $dou_attribute = new Attribute();
        $attribute_list = $dou_attribute->get_attribute_list($item['cat_id'], $item['id']);
        foreach ($attribute_list as $item) {
            $attribute_array[] = $POST['att_' . $item['att_id']];
        }
        $attribute = implode('_', $attribute_array);
    }
    
    // 加入购物车
    $check_stock = $dou_order->check_stock($module, $item_id, $item_number);
    if ($check_stock['code'] == 'without_limit') { // 没有超出购买限制
        // 判断是否已经存在购物车中
        $cart_row = $dou->get_row('order_cart', 'id, item_number', "user_id = '$user_id' AND module = '$module' AND item_id = '$item_id' AND attribute = '$attribute'");
 
        if ($cart_row) { // 已经存在于购物车
            $item_number = $cart_row['item_number'] + $item_number; // 获取商品数量，如果存在于购物车中，则要加上
            $dou->query("UPDATE " . $dou->table('order_cart') . " SET item_number = '$item_number' WHERE id = '$cart_row[id]'");
        } else { // 新加入购物车
            $dou->query("INSERT INTO " . $dou->table('order_cart') . " (id, user_id, module, item_id, item_number, attribute)" . " VALUES (NULL, '$user_id', '$module', '$item_id', '$item_number', '$attribute')");
        }
        
        if ($mode == 'point') {
            $data['mode'] = 'point';
        } else {
            $data['mode'] = 'money';
        }
    } else { // 超出购买限制，弹出提示
        $data['wrong'] = $check_stock['msg'];
    }
}

/**
 * +----------------------------------------------------------
 * 更新购物车商品数量
 * +----------------------------------------------------------
 */
elseif ($rec == 'update') {
    // 初始化
    $data['wrong'] = '';
 
    $action = $check->is_letter($_REQUEST['action']) ? $_REQUEST['action'] : '';
    $id = $check->is_number($_REQUEST['id']) ? $_REQUEST['id'] : '';
 
    if ($action != 'input') {
        $item_number = $dou->get_one("SELECT item_number FROM " . $dou->table('order_cart') . " WHERE id = '$id' AND user_id = '$user_id'");
        if ($action == 'plus') {
            $item_number = $item_number + 1;
        } else {
            $item_number = $item_number - 1;
        }
    } else {
        $item_number = $check->is_number($_POST['item_number']) ? $_POST['item_number'] : 1;
    }
 
    // 因为购物车里不能同时包含两种模块商品，所以直接抽取一条数据读取模块信息
    $module = $dou->get_one("SELECT module FROM " . $GLOBALS['dou']->table('order_cart'));
 
    // 获取购物车项目信息
    $order_cart = $dou->get_row('order_cart', '*', "id = '$id'");
 
    // 购买限制
    $check_stock = $dou_order->check_stock($module, $order_cart['item_id'], $item_number);
    if ($check_stock['code'] == 'without_limit') {
        // 更新商品数量
        $return = $dou->query("UPDATE " . $dou->table('order_cart') . " SET item_number = '$item_number' WHERE id = '$id' AND user_id = '$user_id'");
    } else {
        $data['wrong'] = $check_stock['msg'];
    }
}

/**
 * +----------------------------------------------------------
 * 删除商品
 * +----------------------------------------------------------
 */
elseif ($rec == 'del') {
    $id = $check->is_number($_REQUEST['id']) ? $_REQUEST['id'] : '';
    
    // 删除对应商品
    $dou->query("DELETE FROM " . $dou->table('order_cart') . " WHERE id = '$id' AND user_id = '$user_id'");
}

/**
 * +----------------------------------------------------------
 * 结算页面
 * +----------------------------------------------------------
 */
elseif ($rec == 'checkout') {
    // 获取默认配送方式信息
    $shipping = $dou->get_one("SELECT config FROM " . $GLOBALS['dou']->table('plugin') . " WHERE plugin_group = 'shipping'");
    $shipping = unserialize($shipping);
    
    // 获取购物车信息
    $cart = $dou_order->get_cart($user_id);
 
    // 获取配送方式列表
    $shipping_list = $dou_order->get_shipping_list();
    
    // 免费额度
    if ($shipping['free'] && $cart['item_amount'] >= $shipping['free']) $shipping['fee'] = 0;
    
    // 获取订单信息
    $order = $dou->get_row('user', 'contact, address, phone, postcode', "user_id = '$user_id'");
    $amount['shipping_fee_format'] = $dou->price_format($shipping['fee']);
    $amount['order_amount_format'] = $dou->price_format($cart['item_amount'] + $shipping['fee']);

    // 数据封装
    $data['title'] = $_LANG['order_checkout'];
    $data['cart'] = $cart;
    $data['shipping_list'] = $shipping_list;
    $data['shipping_id'] = $shipping_list[0]['unique_id'];
    $data['order'] = $order;
    $data['amount'] = $amount;
}

/**
 * +----------------------------------------------------------
 * 更改运费和订单总额
 * +----------------------------------------------------------
 */
elseif ($rec == 'change_shipping') {
    $shipping_id = preg_match("/^[A-Za-z0-9_-]+$/", $_REQUEST['shipping_id']) ? $_REQUEST['shipping_id'] : '';
    
    // 获取默认配送方式信息
    $shipping = $dou->get_one("SELECT config FROM " . $GLOBALS['dou']->table('plugin') . " WHERE unique_id = '$shipping_id'");
    $shipping = unserialize($shipping);
    
    // 获取购物车信息
    $cart = $dou_order->get_cart($user_id);
    
    // 免费额度
    if ($shipping['free'] && $cart['item_amount'] >= $shipping['free']) $shipping['fee'] = 0;

    $amount = array (
            "shipping_fee_format" => $dou->price_format($shipping['fee']),
            "order_amount_format" => $dou->price_format($cart['item_amount'] + $shipping['fee'])
    );
    
    // 数据封装
    $data['amount'] = $amount;
}

/**
 * +----------------------------------------------------------
 * 完成订单操作，提交到数据库
 * +----------------------------------------------------------
 */
elseif ($rec == 'checkout_post') {
    $mode = $check->is_letter($_REQUEST['mode']) ? $_REQUEST['mode'] : 'money';
    
    // 初始化
    $data['wrong'] = '';
 
    // 判断是否为空
    $wrong = $check->fn_empty('user', 'contact, phone, address');
 
    // 判断是否含有非法字符
    $wrong = $check->fn_illegal_char('user', 'contact, phone, address, postcode');
    
    // 错误提示
    if ($wrong) {
        foreach ($wrong as $text) {
            $data['wrong'] = $text;
            die(json_encode($data));
        }
    }
 
    // 获取购物车信息
    $cart = $dou_order->get_cart($user_id);
 
    // 检查购物车是否有商品
    if (!is_array($cart)) {
        $data['wrong'] = $_LANG['order_cart_empty'];
        die(json_encode($data));
    }
 
    // 获取和格式化数据
    $order_sn = $dou_order->create_order_sn();
    $module = $cart['module'];
    $order_point = $cart['order_point'];
    $add_time = time();
    $shipping_id = preg_match("/^[A-Za-z0-9_-]+$/", $_REQUEST['shipping_id']) ? $_REQUEST['shipping_id'] : 0;
    $contact = $_POST['contact'];
    $address = $_POST['address'];
    
    // 积分兑换
    if ($mode == 'point') {
        if (!$dou_user->create_point($user_id, 'exchange', $order_point*(-1), $order_sn)) {
            $dou_order->clear_cart();
            $data['wrong'] = $_LANG['point_not_enough'];
            die(json_encode($data));
        }
    }

    // 购买限制
    foreach ($cart['list'] as $item) {
        $check_stock = $dou_order->check_stock($cart['module'], $item['item_id'], $item['number']);
        if ($check_stock['code'] != 'without_limit') {
            $data['wrong'] = $item['name'] . $check_stock['msg'];
            die(json_encode($data));
        }
    }
    
    // 配送方式信息
    $shipping = $dou->get_plugin($shipping_id);
    $shipping_fee = $shipping['config']['fee'];

    // 免费额度
    if ($shipping['config']['free'] && $cart['item_amount'] >= $shipping['config']['free']) $shipping_fee = 0;

    // 计算运费和订单总额
    $order_amount = $cart['item_amount'] + $shipping_fee;
    
    // 安全处理用户输入信息
    $_POST = $firewall->dou_foreground($_POST);
    
    // 订单信息插入
    $dou->query("INSERT INTO " . $dou->table('order') . " (order_id, order_sn, user_id, mode, module, phone, contact, address, postcode, shipping_id, item_amount, shipping_fee, order_amount, order_point, add_time)" . " VALUES (NULL, '$order_sn', '$user_id', '$mode', '$module', '$_POST[phone]', '$contact', '$address', '$_POST[postcode]', '$shipping_id', '$cart[item_amount]', '$shipping_fee', '$order_amount', '$order_point', '$add_time')");

    // 订单商品插入
    $order_id = $dou->insert_id();
    $stock_lock = $_CFG['stock'] && $dou->value_exist('plugin', 'plugin_group', 'payment') ? 1 : 0; // 开启库存功能并且有开启在线支付时才会将库存锁定
    foreach ($cart['list'] as $item) {
        if ($module == 'product')
            $cat_id = $dou->get_value($module, 'cat_id', "id = '$item[id]'");
        
        $dou->query("INSERT INTO " . $dou->table('order_item') . " (id, user_id, order_id, module, cat_id, item_id, name, price, point, item_number, attribute, stock_lock, defined, add_time)" . " VALUES (NULL, '$user_id', '$order_id', '$module', '$cat_id', '$item[item_id]', '$item[name]', '$item[price_normal]', '$item[point]', '$item[number]', '$item[attribute]', '$stock_lock', '$item[defined]', '$add_time')");
    }

    if ($_POST['update_user_information']) {
        $dou->query("UPDATE " . $dou->table('user') . " SET phone = '$_POST[phone]', contact = '$_POST[contact]', address = '$_POST[address]', postcode = '$_POST[postcode]' WHERE user_id = '$user_id'");
    }
    
    // 订单完成，清空购物车
    $dou_order->clear_cart($user_id );
 
    // 如果设置了支付方式，则跳转到收银台
    if ($mode == 'money') {
        $cashier_url = $dou->rewrite_url_miniprogram('order', 'cashier') . '?order_sn=' . $order_sn;
    } else {
        // 如果有安装配送插件则将状态设为1待配送，没有设为10已完成
        $status = $dou->value_exist('plugin', 'plugin_group', 'shipping') ? 1 : 10;
        $dou->query("UPDATE " . $dou->table('order') . " SET status = '$status' WHERE order_sn = '$order_sn'");
     
        $cashier_url = $dou->rewrite_url_miniprogram('user', 'order_list');
    }
 
    // 数据封装
    $data['cashier_url'] = $cashier_url;
}

/**
 * +----------------------------------------------------------
 * 完成订单操作，提交到数据库
 * +----------------------------------------------------------
 */
elseif ($rec == 'success') {
    // 验证并获取合法的ID
    $order_sn = $check->is_number($_REQUEST['order_sn']) ? $_REQUEST['order_sn'] : '';
    
    // 获取订单信息
    $order = $dou->get_row('order', '*', "order_sn = '$order_sn' AND user_id = '$user_id'");
    $order['order_amount_format'] = $dou->price_format($order['order_amount']);

    // 数据封装
    $data['title'] = $_LANG['order_success'];
    $data['order'] = $order;
}

/**
 * +----------------------------------------------------------
 * 收银台，必须要设置了支付方式才会跳转到该页面
 * +----------------------------------------------------------
 */
elseif ($rec == 'cashier') {
    // 验证并获取合法的ID
    $order_sn = $check->is_number($_REQUEST['order_sn']) ? $_REQUEST['order_sn'] : '';
    
    // 获取订单信息
    $order = $dou->get_row('order', '*', "order_sn = '$order_sn' AND user_id = '$user_id' AND status = '0'");
    $order['order_amount_format'] = $dou->price_format($order['order_amount']);

    // 数据封装
    $data['title'] = $_LANG['order_cashier'];
    $data['order'] = $order;
    $data['payment'] = $dou->value_exist('plugin', 'unique_id', 'offlinepay') ? 'offlinepay' : 'wxpay';
}

/**
 * +----------------------------------------------------------
 * 收银台 支付
 * +----------------------------------------------------------
 */
elseif ($rec == 'pay') {
    // 获取订单信息
    $order_sn = $check->is_number($_REQUEST['order_sn']) ? $_REQUEST['order_sn'] : '';
    $order = $dou->get_row('order', '*', "order_sn = '$order_sn' AND status = '0' AND user_id = '$user_id'");
    if ($order) {
        $order['order_amount_format'] = $dou->price_format($order['order_amount']);
        $order['cashier_url'] = $dou->param($_URL['cashier'] . '&order_sn=' . $order['order_sn']);
        $order['pay_time_limit'] = $dou_order->pay_time_limit('', '', true); // 付款超时时间
    }
 
    // 数据封装
    $data['title'] = '离线付款';
    $data['order'] = $order;
}

/**
 * +----------------------------------------------------------
 * 收银台 上传付款凭证
 * +----------------------------------------------------------
 */
elseif ($rec == 'pay_evidence') {
    $data['wrong'] = '';
    
    // 图片上传
    include_once (ROOT_PATH . 'include/file.class.php');
    $file = new File('images/pay/', null, 'jpg,jpeg,gif,png'); // 实例化类文件(文件上传路径，结尾加斜杠)
    
    // 订单权限验证
    $order_sn = $check->is_number($_REQUEST['order_sn']) ? $_REQUEST['order_sn'] : '';
    $order = $dou->get_row('order', '*', "order_sn = '$order_sn' AND user_id = '$user_id'");
    if (!$order) $dou->dou_header($_URL['order_list']);
    
    // 文件上传盒子
    if ($_FILES['pay_evidence']['name'] != "") {
        $pay_evidence = $file->box('order', $order_sn, 'pay_evidence', 'main', '', '', '', 800);
        
        if ($pay_evidence) {
            $pay_time = time();
            $sql = "UPDATE " . $dou->table('order') . " SET pay_id = 'offlinepay', pay_evidence = '$pay_evidence', pay_time = '$pay_time' , status = '-2' WHERE order_sn = '$order_sn' AND user_id = '$user_id'";
            if (!$dou->query($sql))
                $data['wrong'] = '上传失败';
        }
    }
}

echo json_encode($data);

?>